@@ -2,7 +2,7 @@ package Myriad::Config;
 
 use Myriad::Class;
 
-our $VERSION = '0.005'; # VERSION
+our $VERSION = '0.006'; # VERSION
 our $AUTHORITY = 'cpan:DERIV'; # AUTHORITY
 
 =encoding utf8
@@ -362,10 +362,10 @@ async method service_config ($pkg, $service_name) {
 
             # nothing from storage then try other sources
 
-            $value //= $instance_overrides->{$key} ||
-                       $available_config->{$key} ||
-                       $declared_config->{$key}->{default} ||
-                       Myriad::Exception::Config::ConfigRequired->throw(reason => $key);
+            $value //= $instance_overrides->{$key} //
+                $available_config->{$key} //
+                $declared_config->{$key}->{default} //
+                Myriad::Exception::Config::ConfigRequired->throw(reason => $key);
             $value = Myriad::Util::Secret->new($value) if $declared_config->{$key}->{secure};
             $ACTIVE_SERVICES_CONFIG{$storage_request->{key}} = $service_config->{$key} = Ryu::Observable->new($value);
         }
