@@ -2,7 +2,7 @@ package Myriad::RPC::Client::Implementation::Memory;
 
 use Myriad::Class extends => qw(IO::Async::Notifier);
 
-our $VERSION = '0.005'; # VERSION
+our $VERSION = '0.006'; # VERSION
 our $AUTHORITY = 'cpan:DERIV'; # AUTHORITY
 
 =encoding utf8
@@ -81,7 +81,7 @@ async method call_rpc ($service, $method, %args) {
 
     $pending_requests->{$message_id} = $pending;
     await $self->is_started();
-    await $transport->add_to_stream($service, $request->as_hash->%*);
+    await $transport->add_to_stream("service.$service.rpc/$method", $request->as_hash->%*);
 
     try {
         my $message = await Future->wait_any(
