@@ -2,7 +2,7 @@ package Myriad::RPC::Implementation::Memory;
 
 use Myriad::Class extends => qw(IO::Async::Notifier);
 
-our $VERSION = '0.005'; # VERSION
+our $VERSION = '0.006'; # VERSION
 our $AUTHORITY = 'cpan:DERIV'; # AUTHORITY
 
 =encoding utf8
@@ -32,12 +32,13 @@ method group_name { $group_name //= 'processors' }
 
 
 has $should_shutdown;
-has $rpc_methods;
-has $services_list;
+has $rpc_list;
+
+method rpc_list { $rpc_list };
 
 method configure(%args) {
     $transport = delete $args{transport} if exists $args{transport};
-
+    $rpc_list //= [];
     $self->next::method(%args);
 }
 
@@ -53,36 +54,29 @@ async method start () {
     $should_shutdown //= $self->loop->new_future(label => 'rpc::memory::shutdown_future')->without_cancel;
 
     while (1) {
-        if ($services_list && $services_list->@*) {
-            my $service = shift $services_list->@*;
-            push $services_list->@*, $service;
-
-            try {
-                 await $transport->create_consumer_group($service, $self->group_name, 0, 1);
-            } catch {
-                $log->tracef("Group alrady exists");
+        await &fmap_void($self->$curry::curry(async method ($rpc) {
+            unless ($rpc->{group}) {
+                 await $transport->create_consumer_group($rpc->{stream}, $self->group_name, 0, 1);
+                 $rpc->{group} = 1;
             }
 
-            my %messages = await $transport->read_from_stream_by_consumer($service, $self->group_name, hostname());
-            for my $id (sort keys %messages) {
+            my $messages = await $transport->read_from_stream_by_consumer($rpc->{stream}, $self->group_name, hostname());
+            for my $id (sort keys $messages->%*) {
                 my $message;
                 try {
-                    $messages{$id}->{transport_id} = $id;
-                    $message = Myriad::RPC::Message::from_hash($messages{$id}->%*);
-                    if (my $sink = $rpc_methods->{$service}->{$message->rpc}) {
-                        $sink->emit($message);
-                    } else {
-                        Myriad::Exception::RPC::MethodNotFound->throw(reason => $message->rpc);
-                    }
+                    $messages->{$id}->{transport_id} = $id;
+                    $message = Myriad::RPC::Message::from_hash($messages->{$id}->%*);
+                    $rpc->{sink}->emit($message);
                 } catch ($e isa Myriad::Exception::RPC::BadEncoding) {
                     $log->warnf('Recived a dead message that we cannot parse, going to drop it.');
-                    $log->tracef("message was: %s", $messages{$id});
-                    await $self->drop($service, $id);
+                    $log->tracef("message was: %s", $messages->{$id});
+                    await $self->drop($rpc->{stream}, $id);
                 } catch ($e) {
+                    my ($service) = $rpc->{stream} =~ /service.(.*).rpc\//;
                     await $self->reply_error($service, $message, $e);
                 }
             }
-        }
+        }), foreach => [ $self->rpc_list->@* ], concurrent => 8);
         await Future::wait_any($should_shutdown, $self->loop->delay_future(after => 0.1));
     }
 }
@@ -98,9 +92,11 @@ method create_from_sink (%args) {
     my $method = $args{method} // die 'need a method name';
     my $service = $args{service} // die 'need a service';
 
-    push $services_list->@*, $service unless $rpc_methods->{$service};
-
-    $rpc_methods->{$service}->{$method} = $sink;
+    push $rpc_list->@*, {
+        sink => $sink,
+        stream => $self->stream_name($service, $method),
+        group => 0
+    };
 }
 
 =head2 stop
@@ -124,7 +120,7 @@ In this implementation it's done by resolving the L<Future> calling C<done>.
 async method reply_success ($service, $message, $response) {
     $message->response = { response => $response };
     await $transport->publish($message->who, $message->as_json);
-    await $transport->ack_message($service, $self->group_name, $message->transport_id);
+    await $transport->ack_message($self->stream_name($service, $message->rpc), $self->group_name, $message->transport_id);
 }
 
 =head2 reply_error
@@ -138,7 +134,7 @@ In this implementation it's done by resolving the L<Future> calling C<fail>.
 async method reply_error ($service, $message, $error) {
     $message->response = { error => { category => $error->category, message => $error->message, reason => $error->reason } };
     await $transport->publish($message->who, $message->as_json);
-    await $transport->ack_message($service, $self->group_name, $message->transport_id);
+    await $transport->ack_message($self->stream_name($service, $message->rpc), $self->group_name, $message->transport_id);
 }
 
 =head2 drop
@@ -147,8 +143,30 @@ Drop the request because we can't reply to the requester.
 
 =cut
 
-async method drop ($service, $id) {
-    await $transport->ack_message($service, $self->group_name, $id);
+async method drop ($stream, $id) {
+    await $transport->ack_message($stream, $self->group_name, $id);
+}
+
+=head2 stream_name
+
+Get the stream name of the service the current template is
+
+service.$service_name.rpc/$method
+
+it takes:
+
+=over 4
+
+=item * L<service> - the name of service
+
+=item * L<method> - the name of the method
+
+=back
+
+=cut
+
+method stream_name ($service, $method) {
+    return "service.$service.rpc/$method";
 }
 
 1;
