@@ -2,7 +2,7 @@ package Myriad::RPC::Implementation::Redis;
 
 use Myriad::Class extends => qw(IO::Async::Notifier);
 
-our $VERSION = '0.005'; # VERSION
+our $VERSION = '0.006'; # VERSION
 our $AUTHORITY = 'cpan:DERIV'; # AUTHORITY
 
 =encoding utf8
@@ -24,7 +24,7 @@ use Scalar::Util qw(blessed);
 use Myriad::Exception::InternalError;
 use Myriad::RPC::Message;
 
-use constant RPC_SUFFIX => '/rpc';
+use constant RPC_SUFFIX => 'rpc';
 use constant RPC_PREFIX => 'service';
 
 use Exporter qw(import export_to_level);
@@ -42,25 +42,22 @@ method group_name { $group_name }
 has $whoami;
 method whoami { $whoami }
 
-has $rpc_methods;
-has $streams_list;
+has $rpc_list;
+method rpc_list { $rpc_list }
 
 has $running;
 
-sub service_name_from_stream ($stream) {
-    my $pattern = RPC_PREFIX . '\.(.*)' . RPC_SUFFIX . '$';
-    $stream =~ s/$pattern/$1/;
-    return $stream;
-}
-
-sub stream_name_from_service ($service) {
-    return RPC_PREFIX . ".$service" . RPC_SUFFIX;
+sub stream_name_from_service ($service, $method) {
+    return RPC_PREFIX . ".$service.". RPC_SUFFIX . "/$method"
 }
 
 method configure (%args) {
     $redis = delete $args{redis} if exists $args{redis};
     $whoami = hostname();
     $group_name = 'processors';
+    $rpc_list //= [];
+
+    $self->next::method(%args);
 }
 
 async method start () {
@@ -73,64 +70,54 @@ method create_from_sink (%args) {
     my $method = $args{method} // die 'need a method name';
     my $service = $args{service} // die 'need a service name';
 
-    $rpc_methods->{$service}->{$method} = $sink;
-    push $streams_list->@*, {name => stream_name_from_service($service), group => 0};
+    push $rpc_list->@*, {
+        stream => stream_name_from_service($service, $method),
+        sink   => $sink,
+        group  => 0
+    };
 }
 
 async method stop () {
     $running->done unless $running->is_ready;
 }
 
-async method create_group ($stream) {
-    unless ($stream->{group}) {
-        await $self->redis->create_group($stream->{name},$self->group_name);
-        $stream->{group} = 1;
+async method create_group ($rpc) {
+    unless ($rpc->{group}) {
+        await $self->redis->create_group($rpc->{stream}, $self->group_name);
+        $rpc->{group} = 1;
     }
 }
 
 async method listen () {
     return $running //= (async sub {
         while (1) {
-            if ($streams_list && $streams_list->@*) {
-                my $stream = shift $streams_list->@*;
-                push $streams_list->@*, $stream;
-
-                await $self->create_group($stream);
+            await &fmap_void($self->$curry::curry(async method ($rpc) {
+                await $self->create_group($rpc);
 
                 my @items = await $self->redis->read_from_stream(
-                    stream => $stream->{name},
+                    stream => $rpc->{stream},
                     group => $self->group_name,
                     client => $self->whoami
                 );
 
                 for my $item (@items) {
                     push $item->{data}->@*, ('transport_id', $item->{id});
-                    my $service = service_name_from_stream($item->{stream});
                     try {
                         my $message = Myriad::RPC::Message::from_hash($item->{data}->@*);
-                        if (my $sink = $rpc_methods->{$service}->{$message->rpc}) {
-                            $sink->emit($message);
-                        } else {
-                            await $self->reply_error(
-                                $service,
-                                $message,
-                                Myriad::Exception::RPC::MethodNotFound->new(reason => "No such method: " . $message->rpc),
-                            );
-                        }
+                        $rpc->{sink}->emit($message);
                     } catch ($error) {
                         $log->tracef("error while parsing the incoming messages: %s", $error->message);
-                        await $self->drop($_->{service}, $_->{id});
+                        await $self->drop($rpc->{stream}, $_->{id});
                     }
                 }
-            } else {
-                await $self->loop->delay_future(after => 0.001);
-            }
+            }), foreach => [$self->rpc_list->@*], concurrent => 8);
+            await $self->loop->delay_future(after => 0.001);
         }
     })->();
 }
 
 async method reply ($service, $message) {
-    my $stream = stream_name_from_service($service);
+    my $stream = stream_name_from_service($service, $message->rpc);
     try {
         await $self->redis->publish($message->who, $message->as_json);
         await $self->redis->ack($stream, $self->group_name, $message->transport_id);
@@ -150,9 +137,8 @@ async method reply_error ($service, $message, $error) {
     await $self->reply($service, $message);
 }
 
-async method drop ($service, $id) {
+async method drop ($stream, $id) {
     $log->tracef("Going to drop message: %s", $id);
-    my $stream = stream_name_from_service($service);
     await $self->redis->ack($stream, $self->group_name, $id);
 }
 
